function addThelen2003MuscleToModel(model, para, pathpts, pathwraps)
    
    import org.opensim.modeling.*;

    muscle = Thelen2003Muscle(para.name, para.max_isometric_force, para.optimal_fiber_length, ...
        para.tendon_slack_length, para.pennation_angle_at_optimal);
    
    muscle.set_appliesForce(para.appliesforce)
    muscle.set_optimal_force(para.optimal_force);
    muscle.set_max_contraction_velocity(para.max_contraction_velocity);
    muscle.set_FmaxTendonStrain(para.fmaxtendonstrain);

    if isfield(para, 'min_control')
        muscle.set_min_control(para.min_control);
    else
        muscle.set_min_control(0.01);
    end

    if isfield(para, 'max_control')
        muscle.set_max_control(para.max_control);
    else
        muscle.set_max_control(1);
    end
    
    for i = 1 : size(pathpts, 1)
        
        if isfield(pathpts{i, 1}, 'specialized')

            if strcmp (pathpts{i, 1}.specialized, 'MovingPathPoint')

                toadd_moving_pathpoints = MovingPathPoint();
                toadd_moving_pathpoints.setName(pathpts{i, 1}.name);
                toadd_moving_pathpoints.setParentFrame(pathpts{i, 1}.parent_frame)
                toadd_moving_pathpoints.setXCoordinate(model.getCoordinateSet().get(pathpts{i, 1}.socket_x_coordinate));
                toadd_moving_pathpoints.setYCoordinate(model.getCoordinateSet().get(pathpts{i, 1}.socket_y_coordinate));
                toadd_moving_pathpoints.setZCoordinate(model.getCoordinateSet().get(pathpts{i, 1}.socket_z_coordinate));
                
                toadd_moving_pathpoints.set_x_location(pathpts{i, 1}.x_location);
                toadd_moving_pathpoints.set_y_location(pathpts{i, 1}.y_location);
                toadd_moving_pathpoints.set_z_location(pathpts{i, 1}.z_location);
    
                pathpointset = muscle.updGeometryPath().getPathPointSet();
                
                pathpointset.adoptAndAppend(toadd_moving_pathpoints);

            else
                
                error('Unknown Type of PathPoint!')

            end
        
        else
            muscle.addNewPathPoint(pathpts{i}.name, pathpts{i}.body, pathpts{i}.position);
        end
        
    end
    
    for j = 1 : size(pathwraps, 1)
        
        muscle.updGeometryPath().addPathWrap(pathwraps{j}.wrap_object);
        
        pathwrapset = muscle.updGeometryPath().getWrapSet;

        if isfield(pathwraps{j}, 'method')
            pathwrapset.get(j-1).set_method(pathwraps{j}.method);
        else
            pathwrapset.get(j-1).set_method('hybrid');
        end

        if isfield(pathwraps{j}, 'range')

            for k = 1 : length(pathwraps{j}.range)
                pathwrapset.get(j-1).set_range(k-1, pathwraps{j}.range(k));
            end
        
        else
            pathwrapset.get(j-1).set_range(0, -1);
            pathwrapset.get(j-1).set_range(1, -1);
        end

    end

    model.addForce(muscle);

end

