clear; close all;

%% load the library
import org.opensim.modeling.*;

%% prep data
model = Model();

% mass of each link
m1 = 1;
m2 = 1;

% length of each link
l1 = 1;
l2 = 1;

% moment of inertia (Ixx, Iyy, Izz, Ixy, Ixz, Iyz) w.r.t the center of mass
link1_inertia = Inertia(m1*l1^2/12, 0, m1*l1^2/12, 0, 0, 0);
link2_inertia = Inertia(m2*l2^2/12, 0, m2*l2^2/12, 0, 0, 0);

%% create link bodies
link1 = Body('link1', m1, Vec3(0), link1_inertia);
link2 = Body('link2', m2, Vec3(0), link2_inertia);

link1.attachGeometry(Cylinder(0.04, l1/2));
link2.attachGeometry(Cylinder(0.04, l2/2));

model.addBody(link1);
model.addBody(link2);

%% create joints
joint1 = PinJoint('joint1', model.getGround(), Vec3(0, l1 + l2, 0), Vec3(0), link1, Vec3(0, l1/2, 0), Vec3(0));
joint2 = PinJoint('joint2', link1, Vec3(0, -l1/2, 0), Vec3(0), link2, Vec3(0, l2/2, 0), Vec3(0));

model.addJoint(joint1);
joint1.upd_coordinates(0).set_default_value(0);
joint1.upd_coordinates(0).set_range(0, -1.570796326795);
joint1.upd_coordinates(0).set_range(1, 1.570796326795);
joint1.upd_coordinates(0).set_clamped(true);
joint1.upd_coordinates(0).set_locked(false);
joint1.upd_coordinates(0).set_prescribed(false);

model.addJoint(joint2);
joint2.upd_coordinates(0).set_default_value(0);
joint2.upd_coordinates(0).set_range(0, -1.570796326795);
joint2.upd_coordinates(0).set_range(1, 1.570796326795);
joint2.upd_coordinates(0).set_clamped(true);
joint2.upd_coordinates(0).set_locked(false);
joint2.upd_coordinates(0).set_prescribed(false);

%% add wrapping surfaces
wrap1 = WrapCylinder();
wrap1.setName('wrap1');
wrap1.set_active(true);
wrap1.set_xyz_body_rotation(Vec3(0));
wrap1.set_translation(Vec3(0));
wrap1.set_quadrant('x');
wrap1.upd_Appearance().set_visible(true);
wrap1.upd_Appearance().set_opacity(0.5);
wrap1.upd_Appearance().set_color(Vec3(0, 1, 1));
wrap1.upd_Appearance().upd_SurfaceProperties().set_representation(3);
wrap1.set_radius(0.08);
wrap1.set_length(0.1);

link1.addWrapObject(wrap1);

%%
wrap2 = WrapCylinder();
wrap2.setName('wrap2');
wrap2.set_active(true);
wrap2.set_xyz_body_rotation(Vec3(0));
wrap2.set_translation(Vec3(0, -l1/2, 0));
wrap2.set_quadrant('x');
wrap2.upd_Appearance().set_visible(true);
wrap2.upd_Appearance().set_opacity(0.5);
wrap2.upd_Appearance().set_color(Vec3(0, 1, 1));
wrap2.upd_Appearance().upd_SurfaceProperties().set_representation(3);
wrap2.set_radius(0.1);
wrap2.set_length(0.1);

link1.addWrapObject(wrap2);

%%
wrap3 = WrapCylinder();
wrap3.setName('wrap3');
wrap3.set_active(true);
wrap3.set_xyz_body_rotation(Vec3(0));
wrap3.set_translation(Vec3(0, l2/4, 0));
wrap3.set_quadrant('x');
wrap3.upd_Appearance().set_visible(true);
wrap3.upd_Appearance().set_opacity(0.5);
wrap3.upd_Appearance().set_color(Vec3(0, 1, 1));
wrap3.upd_Appearance().upd_SurfaceProperties().set_representation(3);
wrap3.set_radius(0.08);
wrap3.set_length(0.1);

link2.addWrapObject(wrap3);

%%
wrap4 = WrapCylinder();
wrap4.setName('wrap4');
wrap4.set_active(true);
wrap4.set_xyz_body_rotation(Vec3(0));
wrap4.set_translation(Vec3(0));
wrap4.set_quadrant('x');
wrap4.upd_Appearance().set_visible(true);
wrap4.upd_Appearance().set_opacity(0.5);
wrap4.upd_Appearance().set_color(Vec3(0, 1, 1));
wrap4.upd_Appearance().upd_SurfaceProperties().set_representation(3);
wrap4.set_radius(0.08);
wrap4.set_length(0.1);

link2.addWrapObject(wrap4);

%% add muscles
para.appliesforce = true;
para.name = 'muscle1';
para.optimal_force = 1;
para.max_isometric_force = 122.90000000000001;
para.optimal_fiber_length = 0.129;
para.tendon_slack_length = 0.084000000000000005;
para.pennation_angle_at_optimal = 0.12217304764;
para.max_contraction_velocity = 10;
para.fmaxtendonstrain = 0.033000000000000002;

pathpoints = {};
pathpoints{1, 1}.name = 'muscle1-P1';
pathpoints{1, 1}.body = link1;
pathpoints{1, 1}.position = Vec3(0.04, l1/4, 0);

pathpoints{2, 1}.name = 'muscle1-P2';
pathpoints{2, 1}.body = link2;
pathpoints{2, 1}.position = Vec3(0.04, -l2/4, 0);

pathwraps = {};
pathwraps{1, 1}.wrap_object = wrap1;
pathwraps{2, 1}.wrap_object = wrap2;
pathwraps{3, 1}.wrap_object = wrap3;
pathwraps{4, 1}.wrap_object = wrap4;

addThelen2003MuscleToModel(model, para, pathpoints, pathwraps);

%%
model.finalizeConnections();

model.print('double_pendulum.osim');

